@@ -5,9 +5,9 @@ use std::str::from_utf8;
 use pyo3::exceptions::{PyKeyError, PyTypeError, PyValueError};
 use pyo3::ffi;
 use pyo3::intern;
-use pyo3::once_cell::GILOnceCell;
 use pyo3::prelude::*;
-use pyo3::types::{PyDict, PyList, PyString};
+use pyo3::sync::GILOnceCell;
+use pyo3::types::{PyDict, PyList, PyString, PyType};
 use serde::ser::{Error, SerializeMap, SerializeSeq};
 use serde::{Serialize, Serializer};
 
@@ -17,8 +17,8 @@ use crate::build_tools::py_schema_error_type;
 use crate::errors::LocItem;
 use crate::get_pydantic_version;
 use crate::input::InputType;
-use crate::serializers::{SerMode, SerializationState};
-use crate::tools::{safe_repr, SchemaDict};
+use crate::serializers::{DuckTypingSerMode, Extra, SerMode, SerializationState};
+use crate::tools::{safe_repr, write_truncated_to_limited_bytes, SchemaDict};
 
 use super::line_error::ValLineError;
 use super::location::Location;
@@ -26,7 +26,7 @@ use super::types::ErrorType;
 use super::value_exception::PydanticCustomError;
 use super::{InputValue, ValError};
 
-#[pyclass(extends=PyValueError, module="pydantic_core._pydantic_core")]
+#[pyclass(extends=PyValueError, module="pydantic_core._pydantic_core", subclass)]
 #[derive(Clone)]
 #[cfg_attr(debug_assertions, derive(Debug))]
 pub struct ValidationError {
@@ -73,7 +73,7 @@ impl ValidationError {
                                 return cause_problem;
                             }
                         }
-                        PyErr::from_value(err.as_ref(py))
+                        PyErr::from_value_bound(err.into_bound(py).into_any())
                     }
                     Err(err) => err,
                 }
@@ -145,7 +145,7 @@ impl ValidationError {
                     use pyo3::exceptions::PyUserWarning;
 
                     let wrapped = PyUserWarning::new_err((note,));
-                    wrapped.set_cause(py, Some(PyErr::from_value(err.as_ref(py))));
+                    wrapped.set_cause(py, Some(PyErr::from_value_bound(err.clone_ref(py).into_bound(py))));
                     user_py_errs.push(wrapped);
                 }
             }
@@ -167,16 +167,16 @@ impl ValidationError {
             #[cfg(not(Py_3_11))]
             let cause = {
                 use pyo3::exceptions::PyImportError;
-                match py.import("exceptiongroup") {
-                Ok(py_mod) => match py_mod.getattr("ExceptionGroup") {
-                    Ok(group_cls) => match group_cls.call1((title, user_py_errs)) {
-                        Ok(group_instance) => Some(group_instance.into_py(py)),
+                match py.import_bound("exceptiongroup") {
+                    Ok(py_mod) => match py_mod.getattr("ExceptionGroup") {
+                        Ok(group_cls) => match group_cls.call1((title, user_py_errs)) {
+                            Ok(group_instance) => Some(group_instance.into_py(py)),
+                            Err(_) => None,
+                        },
                         Err(_) => None,
                     },
-                    Err(_) => None,
-                },
-                Err(_) => return Some(PyImportError::new_err("validation_error_cause flag requires the exceptiongroup module backport to be installed when used on Python <3.11.")),
-            }
+                    Err(_) => return Some(PyImportError::new_err("validation_error_cause flag requires the exceptiongroup module backport to be installed when used on Python <3.11.")),
+                }
             };
 
             // Set the cause to the ValidationError:
@@ -193,15 +193,31 @@ impl ValidationError {
 
 static URL_ENV_VAR: GILOnceCell<bool> = GILOnceCell::new();
 
-fn _get_include_url_env() -> bool {
-    match std::env::var("PYDANTIC_ERRORS_OMIT_URL") {
-        Ok(val) => val.is_empty(),
-        Err(_) => true,
-    }
-}
-
 fn include_url_env(py: Python) -> bool {
-    *URL_ENV_VAR.get_or_init(py, _get_include_url_env)
+    *URL_ENV_VAR.get_or_init(py, || {
+        // Check the legacy env var first.
+        // Using `var_os` here instead of `var` because we don't care about
+        // the value (or whether we're able to decode it as UTF-8), just
+        // whether it exists (and if it does, whether it's non-empty).
+        match std::env::var_os("PYDANTIC_ERRORS_OMIT_URL") {
+            Some(val) => {
+                // We don't care whether warning succeeded or not, hence the assignment
+                let _ = PyErr::warn_bound(
+                    py,
+                    &py.get_type_bound::<pyo3::exceptions::PyDeprecationWarning>(),
+                    "PYDANTIC_ERRORS_OMIT_URL is deprecated, use PYDANTIC_ERRORS_INCLUDE_URL instead",
+                    1,
+                );
+                // If OMIT_URL exists but is empty, we include the URL:
+                val.is_empty()
+            }
+            // If the legacy env var doesn't exist, check the documented one:
+            None => match std::env::var("PYDANTIC_ERRORS_INCLUDE_URL") {
+                Ok(val) => val == "1" || val.to_lowercase() == "true",
+                Err(_) => true,
+            },
+        }
+    })
 }
 
 static URL_PREFIX: GILOnceCell<String> = GILOnceCell::new();
@@ -225,35 +241,42 @@ fn get_url_prefix(py: Python, include_url: bool) -> Option<&str> {
 
 // used to convert a validation error back to ValError for wrap functions
 impl ValidationError {
-    pub(crate) fn into_val_error(self, py: Python<'_>) -> ValError<'_> {
-        self.line_errors
-            .into_iter()
-            .map(|e| e.into_val_line_error(py))
-            .collect::<Vec<_>>()
-            .into()
+    pub(crate) fn into_val_error(self) -> ValError {
+        self.line_errors.into_iter().map(Into::into).collect::<Vec<_>>().into()
     }
 }
 
 #[pymethods]
 impl ValidationError {
-    #[staticmethod]
+    #[new]
     #[pyo3(signature = (title, line_errors, input_type="python", hide_input=false))]
-    fn from_exception_data(
-        py: Python,
+    fn py_new(title: PyObject, line_errors: Vec<PyLineError>, input_type: &str, hide_input: bool) -> PyResult<Self> {
+        Ok(Self {
+            line_errors,
+            title,
+            input_type: InputType::try_from(input_type)?,
+            hide_input,
+        })
+    }
+
+    #[classmethod]
+    #[pyo3(signature = (title, line_errors, input_type="python", hide_input=false))]
+    fn from_exception_data<'py>(
+        cls: &Bound<'py, PyType>,
         title: PyObject,
-        line_errors: &PyList,
+        line_errors: Bound<'_, PyList>,
         input_type: &str,
         hide_input: bool,
-    ) -> PyResult<Py<Self>> {
-        Py::new(
-            py,
-            Self {
-                line_errors: line_errors.iter().map(PyLineError::try_from).collect::<PyResult<_>>()?,
-                title,
-                input_type: InputType::try_from(input_type)?,
-                hide_input,
-            },
-        )
+    ) -> PyResult<Bound<'py, PyAny>> {
+        cls.call1((
+            title,
+            line_errors
+                .iter()
+                .map(|error| PyLineError::try_from(&error))
+                .collect::<PyResult<Vec<PyLineError>>>()?,
+            InputType::try_from(input_type)?,
+            hide_input,
+        ))
     }
 
     #[getter]
@@ -275,7 +298,7 @@ impl ValidationError {
     ) -> PyResult<Py<PyList>> {
         let url_prefix = get_url_prefix(py, include_url);
         let mut iteration_error = None;
-        let list = PyList::new(
+        let list = PyList::new_bound(
             py,
             // PyList::new takes ExactSizeIterator, so if an error occurs during iteration we
             // fill the list with None before returning the error; the list will then be thrown
@@ -306,9 +329,19 @@ impl ValidationError {
         include_url: bool,
         include_context: bool,
         include_input: bool,
-    ) -> PyResult<&'py PyString> {
-        let state = SerializationState::new("iso8601", "utf8")?;
-        let extra = state.extra(py, &SerMode::Json, true, false, false, true, None);
+    ) -> PyResult<Bound<'py, PyString>> {
+        let state = SerializationState::new("iso8601", "utf8", "constants")?;
+        let extra = state.extra(
+            py,
+            &SerMode::Json,
+            true,
+            false,
+            false,
+            true,
+            None,
+            DuckTypingSerMode::SchemaBased,
+            None,
+        );
         let serializer = ValidationErrorSerializer {
             py,
             line_errors: &self.line_errors,
@@ -335,7 +368,7 @@ impl ValidationError {
             }
         };
         let s = from_utf8(&bytes).map_err(json_py_err)?;
-        Ok(PyString::new(py, s))
+        Ok(PyString::new_bound(py, s))
     }
 
     fn __repr__(&self, py: Python) -> String {
@@ -345,53 +378,22 @@ impl ValidationError {
     fn __str__(&self, py: Python) -> String {
         self.__repr__(py)
     }
-}
 
-// TODO: is_utf8_char_boundary, floor_char_boundary and ceil_char_boundary
-// with builtin methods once https://github.com/rust-lang/rust/issues/93743 is resolved
-// These are just copy pasted from the current implementation
-const fn is_utf8_char_boundary(value: u8) -> bool {
-    // This is bit magic equivalent to: b < 128 || b >= 192
-    (value as i8) >= -0x40
-}
-
-fn floor_char_boundary(value: &str, index: usize) -> usize {
-    if index >= value.len() {
-        value.len()
-    } else {
-        let lower_bound = index.saturating_sub(3);
-        let new_index = value.as_bytes()[lower_bound..=index]
-            .iter()
-            .rposition(|b| is_utf8_char_boundary(*b));
-
-        // SAFETY: we know that the character boundary will be within four bytes
-        unsafe { lower_bound + new_index.unwrap_unchecked() }
+    fn __reduce__<'py>(slf: &Bound<'py, Self>) -> PyResult<(Bound<'py, PyAny>, PyObject)> {
+        let py = slf.py();
+        let callable = slf.getattr("from_exception_data")?;
+        let borrow = slf.try_borrow()?;
+        let args = (
+            borrow.title.bind(py),
+            borrow.errors(py, include_url_env(py), true, true)?,
+            borrow.input_type.into_py(py),
+            borrow.hide_input,
+        )
+            .into_py(slf.py());
+        Ok((callable, args))
     }
 }
 
-pub fn ceil_char_boundary(value: &str, index: usize) -> usize {
-    let upper_bound = Ord::min(index + 4, value.len());
-    value.as_bytes()[index..upper_bound]
-        .iter()
-        .position(|b| is_utf8_char_boundary(*b))
-        .map_or(upper_bound, |pos| pos + index)
-}
-
-macro_rules! truncate_input_value {
-    ($out:expr, $value:expr) => {
-        if $value.len() > 50 {
-            write!(
-                $out,
-                ", input_value={}...{}",
-                &$value[0..floor_char_boundary($value, 25)],
-                &$value[ceil_char_boundary($value, $value.len() - 24)..]
-            )?;
-        } else {
-            write!($out, ", input_value={}", $value)?;
-        }
-    };
-}
-
 pub fn pretty_py_line_errors<'a>(
     py: Python,
     input_type: InputType,
@@ -416,7 +418,7 @@ pub struct PyLineError {
     input_value: PyObject,
 }
 
-impl<'a> IntoPy<PyLineError> for ValLineError<'a> {
+impl IntoPy<PyLineError> for ValLineError {
     fn into_py(self, py: Python<'_>) -> PyLineError {
         PyLineError {
             error_type: self.error_type,
@@ -426,22 +428,22 @@ impl<'a> IntoPy<PyLineError> for ValLineError<'a> {
     }
 }
 
-impl PyLineError {
+impl From<PyLineError> for ValLineError {
     /// Used to extract line errors from a validation error for wrap functions
-    fn into_val_line_error(self, py: Python<'_>) -> ValLineError<'_> {
+    fn from(other: PyLineError) -> ValLineError {
         ValLineError {
-            error_type: self.error_type,
-            location: self.location,
-            input_value: InputValue::PyAny(self.input_value.into_ref(py)),
+            error_type: other.error_type,
+            location: other.location,
+            input_value: InputValue::Python(other.input_value),
         }
     }
 }
 
-impl TryFrom<&PyAny> for PyLineError {
+impl TryFrom<&Bound<'_, PyAny>> for PyLineError {
     type Error = PyErr;
 
-    fn try_from(value: &PyAny) -> PyResult<Self> {
-        let dict: &PyDict = value.downcast()?;
+    fn try_from(value: &Bound<'_, PyAny>) -> PyResult<Self> {
+        let dict = value.downcast::<PyDict>()?;
         let py = value.py();
 
         let type_raw = dict
@@ -449,7 +451,7 @@ impl TryFrom<&PyAny> for PyLineError {
             .ok_or_else(|| PyKeyError::new_err("type"))?;
 
         let error_type = if let Ok(type_str) = type_raw.downcast::<PyString>() {
-            let context: Option<&PyDict> = dict.get_as(intern!(py, "ctx"))?;
+            let context: Option<Bound<'_, PyDict>> = dict.get_as(intern!(py, "ctx"))?;
             ErrorType::new(py, type_str.to_str()?, context)?
         } else if let Ok(custom_error) = type_raw.extract::<PydanticCustomError>() {
             ErrorType::new_custom_error(py, custom_error)
@@ -459,7 +461,7 @@ impl TryFrom<&PyAny> for PyLineError {
             ));
         };
 
-        let location = Location::try_from(dict.get_item("loc")?)?;
+        let location = Location::try_from(dict.get_item("loc")?.as_ref())?;
 
         let input_value = match dict.get_item("input")? {
             Some(i) => i.into_py(py),
@@ -487,7 +489,7 @@ impl PyLineError {
         input_type: InputType,
         include_input: bool,
     ) -> PyResult<PyObject> {
-        let dict = PyDict::new(py);
+        let dict = PyDict::new_bound(py);
         dict.set_item("type", self.error_type.type_string())?;
         dict.set_item("loc", self.location.to_object(py))?;
         dict.set_item("msg", self.error_type.render_message(py, input_type)?)?;
@@ -529,11 +531,12 @@ impl PyLineError {
         write!(output, "  {message} [type={}", self.error_type.type_string())?;
 
         if !hide_input {
-            let input_value = self.input_value.as_ref(py);
+            let input_value = self.input_value.bind(py);
             let input_str = safe_repr(input_value);
-            truncate_input_value!(output, &input_str);
+            write!(output, ", input_value=")?;
+            write_truncated_to_limited_bytes(&mut output, &input_str.to_string(), 50)?;
 
-            if let Ok(type_) = input_value.get_type().name() {
+            if let Ok(type_) = input_value.get_type().qualname() {
                 write!(output, ", input_type={type_}")?;
             }
         }
@@ -575,7 +578,7 @@ struct ValidationErrorSerializer<'py> {
     url_prefix: Option<&'py str>,
     include_context: bool,
     include_input: bool,
-    extra: &'py crate::serializers::Extra<'py>,
+    extra: &'py Extra<'py>,
     input_type: &'py InputType,
 }
 
@@ -607,7 +610,7 @@ struct PyLineErrorSerializer<'py> {
     url_prefix: Option<&'py str>,
     include_context: bool,
     include_input: bool,
-    extra: &'py crate::serializers::Extra<'py>,
+    extra: &'py Extra<'py>,
     input_type: &'py InputType,
 }
 
@@ -637,13 +640,13 @@ impl<'py> Serialize for PyLineErrorSerializer<'py> {
         if self.include_input {
             map.serialize_entry(
                 "input",
-                &self.extra.serialize_infer(self.line_error.input_value.as_ref(py)),
+                &self.extra.serialize_infer(self.line_error.input_value.bind(py)),
             )?;
         }
 
         if self.include_context {
             if let Some(context) = self.line_error.error_type.py_dict(py).map_err(py_err_json::<S>)? {
-                map.serialize_entry("ctx", &self.extra.serialize_infer(context.as_ref(py)))?;
+                map.serialize_entry("ctx", &self.extra.serialize_infer(context.bind(py)))?;
             }
         }
         if let Some(url_prefix) = self.url_prefix {
