@@ -544,8 +544,9 @@ fn check_sub_defaults(
         error: e.to_string(),
         context: None,
     };
-    if let Some(ref default_host) = default_host {
-        if !url.url().has_host() {
+
+    if !url.url().has_host() {
+        if let Some(ref default_host) = default_host {
             url.url_mut().set_host(Some(default_host)).map_err(map_parse_err)?;
         } else if host_required {
             return Err(ErrorType::UrlParsing {
@@ -578,7 +579,7 @@ trait CopyFromPyUrl {
 }
 
 fn get_allowed_schemas(schema: &Bound<'_, PyDict>, name: &'static str) -> PyResult<(AllowedSchemas, String)> {
-    match schema.get_as::<&PyList>(intern!(schema.py(), "allowed_schemes"))? {
+    match schema.get_as::<Bound<'_, PyList>>(intern!(schema.py(), "allowed_schemes"))? {
         Some(list) => {
             if list.is_empty() {
                 return py_schema_err!("`allowed_schemes` should have length > 0");
