@@ -1,11 +1,13 @@
+from __future__ import annotations
+
 import re
 from datetime import date, datetime, time, timedelta, timezone
 from decimal import Decimal
-from typing import Any, Dict
+from typing import Any
 
 import pytest
 
-from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema
+from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema, validate_core_schema
 
 from ..conftest import Err, PyAndJson
 
@@ -22,10 +24,8 @@ from ..conftest import Err, PyAndJson
         pytest.param(1654646400.00, date(2022, 6, 8), id='float'),
         pytest.param('1654646400.00', date(2022, 6, 8), id='float-as-str'),
         pytest.param(Decimal('1654646400'), date(2022, 6, 8), id='decimal'),
-        # (253_402_300_800_000, Err('format YYYY-MM-DD, dates after 9999 are not supported as unix timestamps')),
         pytest.param(253_402_300_800_000, Err('Input should be a valid date'), id='int-too-high'),
-        # (-20_000_000_000, Err('format YYYY-MM-DD, dates before 1600 are not supported as unix timestamps')),
-        pytest.param(-20_000_000_000, Err('Input should be a valid date'), id='int-too-low'),
+        pytest.param(-80_000_000_000_000, Err('Input should be a valid date'), id='int-too-low'),
         pytest.param(datetime(2022, 6, 8), date(2022, 6, 8), id='datetime-exact'),
         pytest.param(
             datetime(2022, 6, 8, 12),
@@ -57,11 +57,13 @@ from ..conftest import Err, PyAndJson
         pytest.param(
             float('-inf'),
             Err(
-                'Input should be a valid date or datetime, dates before 1600 are not supported as unix timestamps '
+                'Input should be a valid date or datetime, dates before 0000 are not supported as unix timestamps '
                 '[type=date_from_datetime_parsing,'
             ),
             id='-inf',
         ),
+        pytest.param('-', Err('Input should be a valid date or datetime, input is too short'), id='minus'),
+        pytest.param('+', Err('Input should be a valid date or datetime, input is too short'), id='pus'),
     ],
 )
 def test_date(input_value, expected):
@@ -183,9 +185,9 @@ def test_date_strict_json_ctx():
             '2000-01-02',
             Err('Input should be less than or equal to 2000-01-01 [type=less_than_equal,'),
         ),
-        ({'lt': '2000-01-01'}, '1999-12-31', date(1999, 12, 31)),
-        ({'lt': '2000-01-01'}, '2000-01-01', Err('Input should be less than 2000-01-01 [type=less_than,')),
-        ({'ge': '2000-01-01'}, '2000-01-01', date(2000, 1, 1)),
+        ({'lt': date(2000, 1, 1)}, '1999-12-31', date(1999, 12, 31)),
+        ({'lt': date(2000, 1, 1)}, '2000-01-01', Err('Input should be less than 2000-01-01 [type=less_than,')),
+        ({'ge': date(2000, 1, 1)}, '2000-01-01', date(2000, 1, 1)),
         (
             {'ge': date(2000, 1, 1)},
             '1999-12-31',
@@ -195,8 +197,8 @@ def test_date_strict_json_ctx():
         ({'gt': date(2000, 1, 1)}, '2000-01-01', Err('Input should be greater than 2000-01-01 [type=greater_than,')),
     ],
 )
-def test_date_kwargs(kwargs: Dict[str, Any], input_value, expected):
-    v = SchemaValidator({'type': 'date', **kwargs})
+def test_date_kwargs(kwargs: dict[str, Any], input_value: date, expected: Err | date):
+    v = SchemaValidator({'type': 'date', **kwargs})  # type: ignore
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_python(input_value)
@@ -207,7 +209,7 @@ def test_date_kwargs(kwargs: Dict[str, Any], input_value, expected):
 
 def test_invalid_constraint():
     with pytest.raises(SchemaError, match=r'date\.gt\n  Input should be a valid date or datetime'):
-        SchemaValidator({'type': 'date', 'gt': 'foobar'})
+        validate_core_schema({'type': 'date', 'gt': 'foobar'})
 
 
 def test_dict_py():
@@ -288,4 +290,4 @@ def test_date_past_future_today():
 
 def test_offset_too_large():
     with pytest.raises(SchemaError, match=r'Input should be less than 86400 \[type=less_than,'):
-        SchemaValidator(core_schema.date_schema(now_op='past', now_utc_offset=24 * 3600))
+        validate_core_schema(core_schema.date_schema(now_op='past', now_utc_offset=24 * 3600))
