@@ -1,4 +1,5 @@
 import re
+from enum import Enum
 
 import pytest
 
@@ -48,36 +49,40 @@ def test_any(py_and_json: PyAndJson, input_value, expected):
 @pytest.mark.parametrize(
     'input_value,expected',
     [
-        ('{"a": 1}', {'a': 1}),
-        (b'{"a": 1}', {'a': 1}),
-        (
+        pytest.param('{"a": 1}', {'a': 1}, id='str'),
+        pytest.param(b'{"a": 1}', {'a': 1}, id='bytes'),
+        pytest.param(
             '🐈 Hello \ud800World',
             Err(
                 'Input should be a valid string, unable to parse raw data as a unicode string '
                 "[type=string_unicode, input_value='🐈 Hello \\ud800World', input_type=str]"
             ),
+            id='str_unicode',
         ),
-        (bytearray(b'{"a": 1}'), {'a': 1}),
-        (
+        pytest.param(bytearray(b'{"a": 1}'), {'a': 1}, id='bytearray'),
+        pytest.param(
             'xx',
             Err(
                 'Invalid JSON: expected value at line 1 column 1 '
                 "[type=json_invalid, input_value='xx', input_type=str]"
             ),
+            id='str_invalid',
         ),
-        (
+        pytest.param(
             b'xx',
             Err(
                 'Invalid JSON: expected value at line 1 column 1 '
                 "[type=json_invalid, input_value=b'xx', input_type=bytes]"
             ),
+            id='bytes_invalid',
         ),
-        (
+        pytest.param(
             bytearray(b'xx'),
             Err(
                 'Invalid JSON: expected value at line 1 column 1 '
                 "[type=json_invalid, input_value=bytearray(b'xx'), input_type=bytearray]"
             ),
+            id='bytearray_invalid',
         ),
     ],
 )
@@ -148,6 +153,23 @@ def test_dict_key(py_and_json: PyAndJson):
     ]
 
 
+def test_enum() -> None:
+    class MyEnum(Enum):
+        a = 'a'
+        b = 'b'
+
+    enum_schema = core_schema.lax_or_strict_schema(
+        core_schema.no_info_after_validator_function(MyEnum, core_schema.str_schema()),
+        core_schema.is_instance_schema(MyEnum),
+    )
+    v = core_schema.json_schema(enum_schema)
+    v = SchemaValidator(v)
+    assert v.validate_python('"a"') == MyEnum.a
+    assert v.validate_python('"b"') == MyEnum.b
+    with pytest.raises(ValidationError):
+        v.validate_python('"c"')
+
+
 def test_any_schema_no_schema():
     v = SchemaValidator(core_schema.json_schema())
     assert 'validator:None' in plain_repr(v)
